import { AxiosWrapper } from '@/utils/axios-wrapper'
import { checkResult } from '@/utils/checkResult'
import { UserStore } from '@/store/user'
import { cloneDeep, omit } from 'lodash-es'
import { translateRouteTitle } from '@/utils/i18n'
import i18n from '@/lang/index'
// 检查路由是否存在
const hasNecessaryRoute = (to, router) => {
  // 将默认注册的路由平铺成一维数组
  const routesOneDimensional = toOneDimensional(router.getRoutes())
  return routesOneDimensional.find((f) => f.path === to.path)
}
// 获取用户信息,返回菜单
const getUserMeta = async (token) => {
  const userStore = UserStore()
  const axios = new AxiosWrapper()
  const result = await axios.$get(`/token/tokenView`, null, {
    headers: {
      token: token
    }
  })
  if (checkResult(result)) {
    userStore.setUser(result.data)
    const resetMenusResult = resetMenus(result.data.menus)
    const storeMenus = toRaw(userStore.menus)
    if (storeMenus.length <= 0) {
      userStore.setMenus(resetMenusResult)
    }
    return result.data.menus
  }
}
// 注册前置守卫
export const registerBeforeRouter = async (router) => {
  router.beforeEach(async (to, from, next) => {
    const token = localStorage.getItem('token')
    if (to.path === '/login') {
      next()
      return
    }
    if (token) {
      if (to.path === '/login') next()
      const menus = await getUserMeta(token)
      // 检查目的地路由是否注册
      const hasRoute = hasNecessaryRoute(to, router)
      if (hasRoute) {
        // 注册了直接进入
        next()
      } else {
        // 没注册就先注册再重定向进入直到进入为止
        await addUserRoutes(menus, router)
        next({ ...to, replace: true })
      }
    } else {
      next()
    }
  })
}
/**
 * 将路由数组一维化
 * @param {Array} routes 路由数组
 * @returns 一维路由数组
 */
const toOneDimensional = (routes) => {
  const result = []
  for (const r of routes) {
    const { children = [], ...others } = r
    result.push(others)
    if (children.length > 0) result.push(...toOneDimensional(children))
  }
  return result
}
// 添加路由
const addUserRoutes = async (menus, router) => {
  return new Promise((resolve, reject) => {
    // 将用户菜单转换成普通对象
    const menuArr = toRaw(menus)
    // 将用户菜单平铺成一维数组,并将目录过滤出去.目录不需要注册,不是组件
    const menuOneDimensional = toOneDimensional(menuArr)
    // 将默认注册的路由平铺成一维数组
    const routesOneDimensional = toOneDimensional(router.getRoutes())
    routesRegister(menuOneDimensional, routesOneDimensional, router)
    resolve()
  })
}
/**
 * 注册路由
 * @param {Array} menus 一维数组菜单
 * @param {Array} defaultRoutes 默认路由一维数组
 * @param {*} router 路由实例
 */
const routesRegister = (menus, defaultRoutes, router) => {
  // 默认注册位置
  const __def = 'Layout'
  const loadComponent = import.meta.glob('../views/**/*.vue')
  for (const route of menus) {
    const { type, route_name, path, component, parent_id, i18n_code } = route
    // 检查路由是否已存在,存在跳过
    const hasRoute = defaultRoutes.find((f) => f.path === path)
    if (hasRoute) continue
    if (type === '0') {
      // 目录, 默认使用 `/${路由名称}`
      const route = {
        path: `/${route_name}`,
        name: route_name
      }
      router.addRoute(__def, route)
    } else if (type === '1' || type === '2') {
      // 菜单 或 子菜单
      const route = {
        path: path,
        name: route_name,
        meta: {
          title: i18n_code
        },
        component: loadComponent[`../views${component}.vue`]
      }
      if (parent_id) {
        const parent = menus.find((f) => f._id === parent_id)
        if (!parent) continue
        const pos = parent.route_name
        router.addRoute(pos, route)
      } else {
        router.addRoute(__def, route)
      }
    }
  }
}

const resetMenus = (menus) => {
  const cMenus = cloneDeep(menus)
  const result = []
  for (const m of cMenus) {
    const mid = omit(m, ['is_use', 'order_num', 'in_admin_frame', 'route_name'])
    const { children } = mid
    if (children) mid.children = resetMenus(children)
    result.push(mid)
  }
  return result
}
